@@ -1,7 +1,7 @@
 from zope.interface import Interface
 
 class IStateMachine(Interface):
-    def add(state, transition_id, newstate, transition_fn):
+    def add(state, transition_id, newstate, transition_fn, **kw):
         """Add a transition to the FSM."""
 
     def execute(context, transition_id):
@@ -11,10 +11,17 @@ class IStateMachine(Interface):
         """ Return the current state of the given object """
 
     def transitions(context, from_state=None):
-        """ Return the available transitions for the given object (defaults
-        to the object's current state) """
+        """ Return the available transition ids for the given object
+        (from_state defaults to the object's current state)"""
 
-    def before_transition(state, newstate, transition_id, context):
+    def transition_info(context, from_state=None):
+        """ Return sequence of dictionaries representing the
+        transition information for context (from_state defaults to the
+        object's current state).  Each dictionary has the keys
+        ``transition_id``, ``from_state``, ``to_state`` as well as any
+        keywords passed in to the ``add`` method for this transition."""
+    
+    def before_transition(state, newstate, transition_id, context, **kw):
         """
         Hook method to be overridden by subclasses (or injected
         directly onto an instance) to allow for before transition
@@ -23,7 +30,7 @@ class IStateMachine(Interface):
         Raise an exception here to abort the transition.
         """
 
-    def after_transition(state, newstate, transition_id, context):
+    def after_transition(state, newstate, transition_id, context, **kw):
         """
         Hook method to be overridden by subclasses (or injected
         directly onto an instance) to allow for after transition
